local AddonName, a = ...
local L = a.Localize
local s = SpellFlashAddon
local c = BittensSpellFlashLibrary

local GetSpellLink = GetSpellLink
local GetTime = GetTime
local GetUnitName = GetUnitName
local SendChatMessage = SendChatMessage
local UnitExists = UnitExists
local UnitInRange = UnitInRange
local UnitIsDeadOrGhost = UnitIsDeadOrGhost
local string = string

a.Rotations = {}
c.RegisterForEvents(a)
s.Spam[AddonName] = function()
	c.Flash(a)
end

local oorTime = {}
local oorCount = {}
local announceTimes = {}
local lastRapture = 0

local function healingNeeded()
	for unit in c.GetGroupMembers() do
		if s.HealthPercent(unit) < 90 and UnitInRange(unit) then
			return true
		end
	end
end

local function getMessage(name)
	local message = a.GetConfig("announcement")
	if name then
		message = string.gsub(message, "<player>", name)
		message = string.gsub(message, "<Player>", name)
		message = string.gsub(message, "<is/are>", L["is"])
	else
		message = string.gsub(message, "<player>", L["you"])
		message = string.gsub(message, "<Player>", L["You"])
		message = string.gsub(message, "<is/are>", L["are"])
	end
	message = string.gsub(
		message, "<Atonement>", GetSpellLink(c.GetID("Atonement")))
	return message
end

local function announceFor(name)
	local now = GetTime()
	local last = announceTimes[name]
	if last ~= nil and now - last < 6 then
		return
	end
	
	if not a.GetConfig("to_raid_off") and s.InParty() then
		SendChatMessage(getMessage(name), "PARTY")
	end
	if a.GetConfig("to_say") then
		SendChatMessage(getMessage(name))
	end
	if a.GetConfig("to_whisper") then
		SendChatMessage(getMessage(), "WHISPER", nil, name)
	end
	announceTimes[name] = now
end

local function announceUnder(percentage)
	for unit in c.GetGroupMembers() do
		local p = s.HealthPercent(unit)
		local name = string.gsub(GetUnitName(unit, true), " ", "", 2)
		if p / 100 < percentage
			and UnitInRange(unit)
			and not UnitIsDeadOrGhost(unit) then
			
			c.Debug("Announce", name, p, "<", percentage)
			local count = oorCount[name]
			if count == nil then
				count = 1
			else
				count = count + 1
			end
			local now = GetTime()
			if count > 2 and now - oorTime[name] < 3 then
				announceFor(name)
			end
			oorCount[name] = count
			oorTime[name] = now
		else
			oorCount[name] = nil
		end
	end
end

a.Rotations.Atonement = {
	Spec = 1,
	
	FlashInCombat = function()
		c.FlashAll("Shadowfiend")
		
		local mana = s.PowerPercent("player")
		if mana < a.GetConfig("conserve_percent") then
			c.FlashAll("Archangel")
			if (GetTime() - lastRapture > 12) then
				c.FlashAll("Power Word: Shield")
			end
		end
		
		if healingNeeded() then
			c.PriorityFlash("Power Infusion", "Holy Fire", "Smite")
			return
		end
		
		if mana < a.GetConfig("heal_only_percent") then
			return
		end
		
		if mana < a.GetConfig("conserve_percent") then
			c.PriorityFlash(
				"Power Infusion", 
				"Holy Fire",
				"Pennance",
				"Smite")
		else
			c.PriorityFlash(
				"Power Infusion", 
				"Holy Fire",
				"Devouring Plague",
				"Pennance at 4",
				"Shadow Word: Pain",
				"Pennance at 2",
				"Mind Blast",
				"Pennance",
				"Smite")
		end
	end,
	
	FlashAlways = function()
		c.FlashAll(
			"Power Word: Fortitude",
			"Shadow Protection",
			"Inner Fire")
		if c.GetBuffDuration("Evangelism") < 5 then
			c.FlashAll("Archangel")
		end
	end,
	
	CastSucceeded = function(info)
		if c.InfoMatches(info, "Power Word: Shield") then
			a.ShieldTarget = info.Target
			c.Debug("Event", "PW:S on", a.ShieldTarget)
		end
	end,
	
	SpellHeal = function(spellID, target, amount)
		if not s.InRaid()
			and spellID == c.GetID("Atonement") 
			and UnitExists(target) then
			
			announceUnder((s.Health(target) - amount) / s.MaxHealth(target))
		end
	end,
	
	Energized = function(spellID)
		if spellID == c.GetID("Rapture") then
			lastRapture = GetTime()
			c.Debug("Event", "Rapture!")
		end
	end,
	
	LeftCombat = function()
		oorTime = {}
		oorCount = {}
		announceTimes = {}
		c.Debug("Event", "Clearing announcement history")
	end
}
